import java.io.*;
import java.util.*;

class Pair<F, S> {
    public F first;
    public S second;
    public Pair(F f, S s) {
        this.first = f;
        this.second = s;
    }
}

class Dsu {
    private int[] p;
    private int[] rank;

    public Dsu(int n) {
        p = new int[n];
        rank = new int[n];
        for (int i = 0; i < n; ++i) p[i] = i;
    }

    public int root(int v) {
        if (v == p[v])
            return v;
        p[v] = root(p[v]);
        return p[v];
    }

    public boolean join(int x, int y) {
        x = root(x);
        y = root(y);
        if (x == y)
            return false;
        if (rank[x] < rank[y]) {
            int tmp = x;
            x = y;
            y = tmp;
        }
        p[y] = x;
        if (rank[x] == rank[y]) ++rank[x];
        return true;
    }
}

class Test {
    private ArrayList<ArrayList<Integer> > g;
    private List<Query> queries = new ArrayList<>();
    private Edge[] edges;
    private int ptr = 0;
    public int n;

    static class Query {
        int t, x, y;
        public Query(int t, int x) {
            this.t = t;
            this.x = x;
        }

        public Query(int t, int x, int y) {
            this.t = t;
            this.x = x;
            this.y = y;
        }
    }

    static class Edge {
        public int from;
        public int to;

        public Edge(int from, int to) {
            this.from = from;
            this.to = to;
        }
    }

    public Test(int n) {
        this.n = n;
        edges = new Edge[n - 1];
        g = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            g.add(new ArrayList<>());
    }

    public Edge[] getEdges() {
        if (ptr != g.size() - 1)
            throw new RuntimeException("Incorrect structure of tree!");
        return edges;
    }

    public int getNumVertexes() {
        return g.size();
    }

    public void addEdge(int from, int to) {
        edges[ptr++] = new Edge(from, to);
        g.get(from).add(to);
        g.get(to).add(from);
    }

    public ArrayList<Integer> getAdjacent(int v) {
        return g.get(v);
    }

    public void addQuery(Query query) {
        queries.add(query);
    }

    public List<Query> getQueries() {
        return queries;
    }
}

public class TestGen {

    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0xCEEECCEE);

    public static final int GROUP2_N = 300;
    public static final int GROUP3_N = 2000;
    public static final int MAX_N = 100_000;//10^5
    public static final int MAX_VAL = 1_000_000_000 - 1;
    public static final int MAX_LEN = 9;

    private String folder;
    private String subtask;
    private int groupNo;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void setGroupNo(int groupNo) {
        this.groupNo = groupNo;
    }

    void print(Test test) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        System.out.println(testCount);
        Test.Edge[] ed = test.getEdges();
        if (ed.length != test.getNumVertexes() - 1)
            throw new RuntimeException("Wrong number of edges!");
        ArrayList<Test.Edge> edges = new ArrayList<>();
        if (!isSamples)
            test = coloringTree(test);

        Collections.addAll(edges, ed);
        if (!isSamples) {
            for (Test.Edge e : edges)
                if (random.nextBoolean()) {
                    int tmp = e.from;
                    e.from = e.to;
                    e.to = tmp;
                }
        } else
            for (Test.Edge e : edges) {
                e.from++;
                e.to++;
            }
        out.println(test.getNumVertexes());
        for (int i = 0; i < edges.size(); ++i)
            if (i + 1 == edges.size())
                out.println(edges.get(i).from + " " + edges.get(i).to);
            else
                out.println(edges.get(i).from + " " + edges.get(i).to);
        List<Test.Query> queries = test.getQueries();
        out.println(queries.size());
        for (Test.Query q : queries)
            if (q.t == 1)
                out.println("1 " + q.x);
            else
                out.println("2 " + q.x + " " + q.y);
        out.close();

        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, ""));
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");
        isSamples = true;

        System.out.println("==== Generating samples ===");
        Test test1 = new Test(6);
        test1.addEdge(2, 4);
        test1.addEdge(4, 5);
        test1.addEdge(4, 0);
        test1.addEdge(2, 3);
        test1.addEdge(3, 1);
        test1.addQuery(new Test.Query(1, 5));
        test1.addQuery(new Test.Query(1, 4));
        test1.addQuery(new Test.Query(2, 1, 2));
        test1.addQuery(new Test.Query(2, 6, 3));
        test1.addQuery(new Test.Query(1, 4));
        test1.addQuery(new Test.Query(2, 1, 3));
        print(test1);

        Test test2 = new Test(8);
        test2.addEdge(1, 0);
        test2.addEdge(0, 6);
        test2.addEdge(1, 3);
        test2.addEdge(2, 5);
        test2.addEdge(1, 2);
        test2.addEdge(5, 7);
        test2.addEdge(3, 4);

        test2.addQuery(new Test.Query(1, 7));
        test2.addQuery(new Test.Query(1, 6));
        test2.addQuery(new Test.Query(1, 3));
        test2.addQuery(new Test.Query(2, 8, 5));
        test2.addQuery(new Test.Query(2, 7, 6));
        test2.addQuery(new Test.Query(2, 1, 1));
        print(test2);

        testCount = 0;
        isSamples = false;
    }

    public Test coloringTree(Test test) {
        int n = test.getNumVertexes();
        List<Integer> perm = new ArrayList<>();
        for (int i = 1; i < n; ++i)
            perm.add(i);
        Collections.shuffle(perm, random);
        perm.add(0, 0);

        int[] inv = new int[n];
        for (int i = 0; i < perm.size(); ++i)
            inv[perm.get(i)] = i + 1;
        Test.Edge[] edges = test.getEdges();
        for (Test.Edge e : edges) {
            e.from = inv[e.from];
            e.to = inv[e.to];
        }
        List<Test.Query> queries = test.getQueries();
        for (Test.Query q : queries)
            if (q.t == 1) q.x = inv[q.x - 1];
            else {
                q.x = inv[q.x - 1];
                q.y = inv[q.y - 1];
            }
        return test;
    }

    public Test genBinaryStructure(int n) {
        Test test = new Test(n);
        int i;
        for (i = 0; 2 * i + 2 < n; ++i) {
            test.addEdge(i, 2 * i + 1);
            test.addEdge(i, 2 * i + 2);
        }
        i = 2 * i + 1;
        for (; i < n; ++i)
            test.addEdge(random.nextInt(i), i);
        return test;
    }

    public Test genRandomWithBigDiameter(int n) {
        Test test = new Test(n);
        Dsu dsu = new Dsu(n);
        int edges = 0;
        while (edges < n - 1) {
            int x = random.nextInt(n);
            int y = random.nextInt(n);
            if (dsu.join(x, y)) {
                ++edges;
                test.addEdge(x, y);
            }
        }
        return test;
    }

    public Test genRandomWithManyListsStructure(int n) {
        Test test = new Test(n);
        for (int i = 1; i < n; ++i)
            test.addEdge(random.nextInt(i), i);
        return test;
    }

    public Test genLongTreeStructure(int n, int ridgeLen) {
        Test test = new Test(n);
        for (int i = 0; i + 1 < ridgeLen; ++i)
            test.addEdge(i, i + 1);
        for (int i = ridgeLen; i < n; ++i)
            test.addEdge(random.nextInt(i), i);
        return test;
    }

    public Test genNBranchesStructure(int n, int branches) {
        if (n - 1 < branches)
            throw new RuntimeException("Too much branches!");
        Test test = new Test(n);
        int len = (n - 1) / branches;
        int v = 1;
        for (int i = 0; i < branches && v < n; ++i) {
            test.addEdge(0, v++);
            for (int j = 1; j < len && v < n; ++j, ++v)
                test.addEdge(v - 1, v);
        }
        if (v < n) {
            test.addEdge(v, len);
            ++v;
            while (v < n)
                test.addEdge(v - 1, v++);
        }
        return test;
    }

    public Test genBoneStructure(int middle, int left, int right) {
        Test test = new Test(middle + left + right);
        for (int i = 0; i + 1 < middle; ++i)
            test.addEdge(i, i + 1);
        for (int i = middle; i < middle + left; ++i)
            test.addEdge(0, i);
        for (int i = middle + left; i < middle + left + right; ++i)
            test.addEdge(middle - 1, i);
        return test;
    }

    void genHandleTests() throws IOException {
    }

    public void genSubtask(int n, int m, int groupNo, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        setGroupNo(groupNo);
        System.out.println("==== Generating group #" + groupNo + " ===");
        int sq = (int)Math.sqrt(n);

        if (groupNo == 1) {
            print(genSequenceQueries(1, m, 0));
            print(genSequenceQueries(2, m, 0));
            print(genSequenceQueries(3, m, 0));

            print(genSequenceQueries(n, m, 0));
            print(genSequenceQueries(n, m, 0.2));
            print(genSequenceQueries(n, m, 0.5));

            print(randomQueries(genLongTreeStructure(n, n), m, 0.5));
            print(randomQueries(genLongTreeStructure(n, n), m, 0.7));
            print(randomQueries(genLongTreeStructure(n, n), m, 0.9));
            print(randomQueries(genLongTreeStructure(n, n), m, 0.2));
            print(randomQueries(genLongTreeStructure(n, n), m, 0.0));
            print(randomQueries(genLongTreeStructure(n, n), m, 0.1));
            print(randomQueries(genLongTreeStructure(n, n), m, 1.0));

            print(addDelQueries(genLongTreeStructure(n, n), 3 * m / 4));

        } else {

            print(randomQueries(genLongTreeStructure(1, 1), m));
            print(randomQueries(genLongTreeStructure(2, 2), m));
            print(randomQueries(genLongTreeStructure(3, 3), m));

            print(randomQueries(genBinaryStructure(n), m));
            print(randomQueries(genNBranchesStructure(n, n / 2), m));
            print(randomQueries(genNBranchesStructure(n, sq), m));
            print(randomQueries(genRandomWithBigDiameter(n), m));
            print(randomQueries(genRandomWithManyListsStructure(n), m));
            print(randomQueries(genLongTreeStructure(n, n / 2), m));
            print(randomQueries(genLongTreeStructure(n, n), m));
            print(randomQueries(genBoneStructure(n / 2, n / 4, n / 4), m));

            print(addDelQueries(genLongTreeStructure(n, n), 3 * m / 4));

            if (groupNo != 2) {
                print(randomQueries(genBinaryStructure(n), m, 0.3));
                print(randomQueries(genBinaryStructure(n), m, 0.8));
                print(randomQueries(genRandomWithBigDiameter(n), m, 0.8));
                print(randomQueries(genRandomWithBigDiameter(n), m, 0.1));
                print(randomQueries(genRandomWithManyListsStructure(n), m));
                print(randomQueries(genLongTreeStructure(n, n / 2), m, 0.3));
                print(randomQueries(genBoneStructure(n / 3, n / 3, n / 3), m, 0.2));
                print(randomQueries(genBoneStructure(n / 2, n / 4, n / 4), m, 0.9));
            }
        }

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d, m &lt;= %d\"", subtaskCount, n, m));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test addDelQueries(Test test, int m) {
        int n = test.n;
        while (m / 4 > 0) {
            int cn = randInt(m / 4);
            List<Integer> ct = new ArrayList<>();
            for (int j = 0; j < 2 * cn;)
                if (random.nextDouble() < 0.5) {
                    int x, y;
                    if (random.nextDouble() < 0.1) {
                        x = randInt(n);
                        y = x;
                    }
                    else {
                        x = randInt(n);
                        y = randInt(n);
                    }
                    test.addQuery(new Test.Query(2, x, y));
                    --m;
                } else if (j < cn) {
                    int x = randInt(n);
                    test.addQuery(new Test.Query(1, x));
                    --m;
                    ct.add(x);
                    ++j;
                } else {
                    int x = ct.get(j - cn);
                    test.addQuery(new Test.Query(1, x));
                    --m;
                    ++j;
                }
        }
        return test;
    }

    private Test genSequenceQueries(int n, int m, double prob) {
        Test test = new Test(n);
        for (int i = 0; i + 1 < n; ++i)
            test.addEdge(i, i + 1);
        for (int i = 0; i < n; ++i) {
            if (random.nextDouble() < 0.5) {
                if (random.nextDouble() < prob)
                    test.addQuery(new Test.Query(1, randInt(i + 1)));
                else
                    test.addQuery(new Test.Query(1, i + 1));
            } else
                test.addQuery(new Test.Query(2, randInt(n), randInt(n)));
        }
        return test;
    }

    private Test randomQueries(Test test, int m) {
        return randomQueries(test, m, 0.5);
    }

    private Test randomQueries(Test test, int m, double prob) {
        for (int i = 0; i < m; ++i) {
            if (random.nextDouble() < prob)
                test.addQuery(new Test.Query(1, randInt(test.n)));
            else
                test.addQuery(new Test.Query(2, randInt(test.n), randInt(test.n)));
        }
        return test;
    }

    private int randInt(int n) {
        return random.nextInt(n) + 1;
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(MAX_N, MAX_N, 1, 19);
        genSubtask(GROUP2_N, GROUP2_N, 2, 18);
        genSubtask(GROUP3_N, GROUP3_N, 3, 28);
        genSubtask(MAX_N, MAX_N, 4, 35);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}